package com.muzhi.zgd.http;

import com.muzhi.zgd.exception.HttpResponseException;
import com.muzhi.zgd.handler.PlainTextResponseHandler;
import com.muzhi.zgd.handler.ResponseHandler;
import com.muzhi.zgd.oauth2.ApiClientConfig;
import com.muzhi.zgd.oauth2.ApiToken;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

public class ApiHttpClientUtils {

	public static ResponseHandler<String> textHandler = new PlainTextResponseHandler();

	private static ApiHttpClientUtils client;

	private ApiHttpClientUtils() {
	}

	public static synchronized ApiHttpClientUtils get() {
		if (client == null) {
			client = new ApiHttpClientUtils();
		}
		return client;
	}
	
	public static synchronized ApiHttpClientUtils get(ApiClientConfig config) {
		if (client == null) {
			ApiToken.get().set(config);
			client = new ApiHttpClientUtils();
		}
		return client;
	}
	
	/**
	 * 
	 * @description	： Get方式调用API带认证权限
	 * @author 		： 万大龙（743）
	 * @date 		：2017年10月27日 上午10:52:20
	 * @param baseURL
	 * @param paramsMap
	 * @return
	 * @throws IOException
	 */
	public String invokeApiWithGet(String baseURL, Map<String, Object> paramsMap) throws HttpResponseException, IOException {
		Map<String, String> headers = new HashMap<String, String>();
		headers.put(HttpHeaders.AUTHORIZATION, "Bearer " + ApiToken.get().getAccessToken());
		return HttpConnectionUtils.httpRequestWithGet(baseURL, paramsMap, ContentType.UTF_8, headers, textHandler);
	}
	
	/**
	 * 
	 * @description	： Post方式调用API带认证权限
	 * @author 		： 万大龙（743）
	 * @date 		：2017年10月27日 上午10:52:46
	 * @param baseURL
	 * @param paramsMap
	 * @return
	 * @throws IOException
	 */
	public String invokeApiWithPost(String baseURL, Map<String, Object> paramsMap) throws HttpResponseException, IOException {
		Map<String, String> headers = new HashMap<String, String>();
		headers.put(HttpHeaders.AUTHORIZATION, "Bearer " + ApiToken.get().getAccessToken());
		return HttpConnectionUtils.httpRequestWithPost(baseURL, paramsMap, ContentType.UTF_8, ContentType.APPLICATION_FORM_URLENCODED + "; charset=" + ContentType.UTF_8, headers, textHandler);
	}
	
	/**
	 * 
	 * @description	： Get方式调用API不带认证权限
	 * @author 		： 樊新亮（1505）
	 * @date 		：2017年11月9日 上午11:08:34
	 * @param baseURL
	 * @param paramsMap
	 * @return
	 * @throws IOException
	 */
	public String invokeApiWithGetNotToken(String baseURL, Map<String, Object> paramsMap) throws IOException {
		Map<String, String> headers = new HashMap<String, String>();
		return HttpConnectionUtils.httpRequestWithGet(baseURL, paramsMap, ContentType.UTF_8, headers, textHandler);
	}
	
	/**
	 * 
	 * @description	： Post方式调用API不带认证权限
	 * @author 		： 樊新亮（1505）
	 * @date 		：2017年11月9日 上午11:08:55
	 * @param baseURL
	 * @param paramsMap
	 * @return
	 * @throws IOException
	 */
	public String invokeApiWithPostNotToken(String baseURL, Map<String, Object> paramsMap) throws IOException {
		Map<String, String> headers = new HashMap<String, String>();
		return HttpConnectionUtils.httpRequestWithPost(baseURL, paramsMap, ContentType.UTF_8, ContentType.APPLICATION_FORM_URLENCODED + "; charset=" + ContentType.UTF_8, headers, textHandler);
	}
	
}
