/**
 * <p>Coyright (R) 2014 正方软件股份有限公司。<p>
 */
package com.muzhi.zgd.oauth2;

import com.alibaba.fastjson.JSONObject;
import com.muzhi.zgd.exception.HttpResponseException;
import com.muzhi.zgd.handler.PlainTextResponseHandler;
import com.muzhi.zgd.handler.ResponseHandler;
import com.muzhi.zgd.http.ContentType;
import com.muzhi.zgd.http.HttpConnectionUtils;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

/**
 * 
 * @className	： Token
 * @description	： TODO(描述这个类的作用)
 * @author 		：樊新亮（1505）
 * @date		： 2017年9月23日 下午8:29:20
 * @version 	V1.0
 */
public class ApiToken{
	
	public static ResponseHandler<String> textHandler = new PlainTextResponseHandler();

	private static ApiToken token;
	private String accessToken;
	private Long tokenExpiryTime;
	private ApiClientConfig config;
	
	private ApiToken() {}
	
	public static synchronized ApiToken get() {
		if(token == null) {
			token = new ApiToken();
		}
		return token;
	}
	
	public ApiToken set(ApiClientConfig config) {
		this.config = config;
		return token;
	}
	
	public String getAccessToken() throws HttpResponseException, IOException{
		if(accessToken == null && tokenExpiryTime == null) {
			refreshToken();
		}
		if(System.currentTimeMillis() >= tokenExpiryTime - 10000) {
			refreshToken();
		}
		return accessToken;
	}

	/**
	 * 刷新令牌
	 * @description	： TODO
	 * @author 		： 樊新亮（1505）
	 * @date 		：2017年9月23日 下午9:08:34
	 * @return
	 */
	public ApiToken refreshToken() throws HttpResponseException, IOException{

		Map<String,Object> paramsMap = new HashMap<String, Object>();
		paramsMap.put("appKey", this.config.getAppKey());
		paramsMap.put("appSecret", this.config.getAppSecret());
		
		/**获取token*/
		String text = HttpConnectionUtils.httpRequestWithPost(this.config.getAccessTokenUrl(), paramsMap, ContentType.UTF_8 , textHandler);
		JSONObject ret = JSONObject.parseObject(text);
		if("success".equalsIgnoreCase(ret.getString("status"))) {
			accessToken = ret.getJSONObject("token").getString("accessToken");
			Long expiresIn = ret.getJSONObject("token").getLong("tokenExpiryTime");
			tokenExpiryTime = System.currentTimeMillis() + expiresIn * 1000;
		} else if("fail".equalsIgnoreCase(ret.getString("status"))){
			throw new HttpResponseException(ret.getString("token"));
		}
		
		return token;
	}
	
}
